#include <psychlops.h>
using namespace Psychlops;


const int COLOR_SCALE = 1023;


double estimate_gamma(double colorlevel) {
	double estimated_gamma=0.0, gamma_step=8.0;
	while(0.0001<gamma_step) {
		while(0<pow(colorlevel,estimated_gamma)-0.5) {
			estimated_gamma += gamma_step;
		}
		estimated_gamma -= gamma_step;
		gamma_step /= 2.0;
	}
	estimated_gamma += gamma_step;
	return estimated_gamma;
}

void measure_gamma() {
	Canvas display(Canvas::fullscreen);

	double measure_r=0.5,measure_g=0.5, measure_b=0.5;
	double gammar=1.0,gammag=1.0, gammab=1.0;

	Color::setGammaValue(1.0,1.0,1.0);

	Psychlops::Rectangle center_rect(100,100), grads(200,1);

	Range rng;
	Independent << measure_r | "red"   | 0<=rng<=2.0 | 1.0/32 | 1.0/1024;
	Independent << measure_g | "green" | 0<=rng<=2.0 | 1.0/32 | 1.0/1024;
	Independent << measure_b | "blue"  | 0<=rng<=2.0 | 1.0/32 | 1.0/1024;
	Independent << gammar | "gamma red"   | 0<=rng<=2.0 | 1.0/32 | 1.0/1024;
	Independent << gammag | "gamma green" | 0<=rng<=2.0 | 1.0/32 | 1.0/1024;
	Independent << gammab | "gamma blue"  | 0<=rng<=2.0 | 1.0/32 | 1.0/1024;

	while(!Input::get(Keyboard::esc)) {
		display.clear();

		center_rect.centering().shift(-300,0);
		grads.centering().shift(-300,-100);
		for(int i=0; i<200; i++) grads.shift(0,1).draw(Color((double)(i%2),0.0,0.0));
		center_rect.draw(Color(measure_r,0.0,0.0));

		center_rect.shift(+300,0);
		grads.centering().shift(   0,-100);
		for(int i=0; i<200; i++) grads.shift(0,1).draw(Color(0.0,(double)(i%2),0.0));
		center_rect.draw(Color(0.0,measure_g,0.0));

		center_rect.shift(+300,0);
		grads.centering().shift(+300,-100);
		for(int i=0; i<200; i++) grads.shift(0,1).draw(Color(0.0,0.0,(double)(i%2)));
		center_rect.draw(Color(0.0,0.0,measure_b));

		gammar = estimate_gamma(measure_r);
		gammag = estimate_gamma(measure_g);
		gammab = estimate_gamma(measure_b);
		display.flip();
	}

}


void psychlops_main() {

	Procedure p;
	p.setDesign(Procedure::DEMO);
	p.setProcedure(measure_gamma);
	p.run();
}
